/*
 *  <legal_notice>
 *   MaxLinear, Inc. retains all right, title and interest (including all intellectual
 *   property rights) in and to this computer program, which is protected by applicable
 *   intellectual property laws.  Unless you have obtained a separate written license from
 *   MaxLinear, Inc. or an authorized licensee of MaxLinear, Inc., you are not authorized
 *   to utilize all or a part of this computer program for any purpose (including
 *   reproduction, distribution, modification, and compilation into object code), and you
 *   must immediately destroy or return all copies of this computer program.  If you are
 *   licensed by MaxLinear, Inc. or an authorized licensee of MaxLinear, Inc., your rights
 *   to utilize this computer program are limited by the terms of that license.
 *  
 *   This computer program contains trade secrets owned by MaxLinear, Inc. and, unless
 *   authorized by MaxLinear, Inc. in writing, you agree to maintain the confidentiality
 *   of this computer program and related information and to not disclose this computer
 *   program and related information to any other person or entity.
 *  
 *   Misuse of this computer program or any information contained in it may results in
 *   violations of applicable law.  MaxLinear, Inc. vigorously enforces its copyright,
 *   trade secret, patent, contractual, and other legal rights.
 *  
 *   THIS COMPUTER PROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTIES, AND 
 *    MAXLINEAR, INC.
 *   EXPRESSLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING THE 
 *   WARRANTIES OF
 *   MERCHANTIBILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND 
 *   NONINFRINGEMENT.
 *  
 *  ***************************************************************************************
 *                                          Copyright (c) 2016/2022, MaxLinear, Inc.
 *  ***************************************************************************************
 *  </legal_notice>
 */


package resources;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Hashtable;

public class ParseHtmlDescription {

  static Hashtable<String, String> list;
  static final String columns[] = {"Description", "Type", "Indexed", "Read", "Write", "Stored", "Auto-stored", "API Set-only", "Default value", "First boot value", "Factory reset value", "Units", "Range"};

  static ArrayList<String> parameters;
  
  public enum columnType {
    LONG_TEXT, SHORT_TEXT, BOOLEAN 
  }
  static final columnType columnsType[] = {columnType.LONG_TEXT, columnType.SHORT_TEXT, columnType.BOOLEAN, columnType.BOOLEAN, columnType.BOOLEAN, columnType.BOOLEAN, columnType.BOOLEAN, columnType.BOOLEAN, columnType.SHORT_TEXT, columnType.SHORT_TEXT, columnType.SHORT_TEXT, columnType.SHORT_TEXT, columnType.SHORT_TEXT};

  static public void init(InputStream configStream)
  {
    list = new Hashtable<String, String>();
    parameters = new ArrayList<String>();
    try
    {
      BufferedReader bfr = new BufferedReader(new InputStreamReader(configStream, "UTF-8"));
      String line;
      while ((line = bfr.readLine()) != null)
      {
        line = line.replace("<br>", "\n");
        line = line.replace("<BR>", "\n");
        int index = -1;
        int column = -1;
        String param = "";
        do
        {
          index = line.indexOf("</td>");
          if (index != -1)
          {
            int start = line.indexOf(">");
            if (start > 0 && start < index)
            {
              String part = line.substring(start+1, index);
              part = part.trim();
              if (column == -1)
              {
                int bracket = part.indexOf("[");
                if (bracket > 0)
                {
                  part = part.substring(0, bracket);
                }
                param = part;
                parameters.add(param);
                column = 0;
              }
              else
              {
                list.put(param+"."+columns[column], part);
                column++;
              }
            }
            else
            {
              column = -1;
            }
            line = line.substring(index+5, line.length());
          }
          else
          {
            column = -1;
          }
        }
        while (index != -1);
      }
      bfr.close();
    } catch (Exception e)
    {
      e.printStackTrace();
    }
  }
  static public String getHelp(String param)
  {
    param = removeIndex(param);

    String help;
    if (list.containsKey(param+".Description"))
    {
      help = "<h2>"+param+"</h2><br>";
      for (int i = 0; i < columns.length; i++)
      {
        String s = (String) list.get(param+"."+columns[i]);
        if (s != null)
        {
          s = s.replace("\n", "<br>");
          switch (columnsType[i])
          {
          case LONG_TEXT:
            help += s+"<br><br>";
            break;
          case SHORT_TEXT:
            help += "<b>"+columns[i]+"</b>: "+s+"<br>";
            break;
          case BOOLEAN:
            if (s.contains("*"))
            {
              s = "YES";
            }
            else
            {
              s = "NO";
            }
            help += "<b>"+columns[i]+"</b>: "+s+"<br>";
            break;
          }
        }
      }
      help = "<html>"+help+"</html>";

      return help;
    }
    else
    {
      return "No information";
    }
  }
  static public boolean isEditable(String param) {
    param = removeIndex(param);
    if (list.containsKey(param+".Write") && list.containsKey(param+".API Set-only"))
    {
      String s1 = (String) list.get(param+".Write");
      String s2 = (String) list.get(param+".API Set-only");
      if (s1.contains("*") && !s2.contains("*"))
      {
        return true;
      }
      else
      {
        return false;
      }
    }
    return true;
  }
  public static String getDescription(String param) {
    param = removeIndex(param);
    if (list.containsKey(param+".Description"))
    {
      String s = (String) list.get(param+".Description");
      return s;
    }
    return "";
  }
  public static String getType(String param) {
    String paramaux = removeIndex(param);
    if (list.containsKey(paramaux+".Type"))
    {
      String s = (String) list.get(paramaux+".Type");
      if (paramaux.equals(param))
      {
        return s;
      }
      else
      {
        s = "1"+s.substring(s.indexOf("x"));
        return s;
      }
    }
    return "Unknown";
  }
  private static String removeIndex(String param)
  {
    if (param.contains("."))
    {
      try
      { 
        String index = param.substring(param.lastIndexOf(".")+1, param.length());
        Integer.parseInt(index);
        param = param.substring(0, param.lastIndexOf("."));
        return removeIndex(param);
      }
      catch (Exception e)
      {
        return param;
      }
    }
    else 
    {
      return param;
    }
  }
  public static String getRange(String param) {
    param = removeIndex(param);
    if (list.containsKey(param+".Range"))
    {
      String s = (String) list.get(param+".Range");
      return s;
    }
    return "Unknown";
  }
  public static String[] getParams() {
    return parameters.toArray(new String[parameters.size()]);
    // TODO Auto-generated method stub
  }
  public static boolean isArray(String param) {
    param = removeIndex(param);
    String type = (String) list.get(param+".Type");
    if (type == null || type.startsWith("1x"))
    {
      return false;
    }
    else if (type.contains("xCHAR"))
    {
      return false;
    }
    else
    {
      return true;
    }
  }
}
